/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.wayang.core.api.configuration;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.wayang.core.api.Configuration;

import java.util.Collection;
import java.util.Iterator;

/**
 * Provides a {@link Collection} of objects.
 */
public abstract class CollectionProvider<Value> implements Iterable<Value> {

    private static final Logger logger = LogManager.getLogger();

    protected CollectionProvider<Value> parent;

    protected final Configuration configuration;

    public CollectionProvider(Configuration configuration) {
        this(configuration, null);
    }

    public CollectionProvider(Configuration configuration, CollectionProvider parent) {
        this.configuration = configuration;
        this.parent = parent;
    }


    public void setParent(CollectionProvider<Value> parent) {
        this.parent = parent;
    }

    /**
     * Provide all objects.
     *
     * @return the provided objects from this instance and its parents (unless those are masked).
     */
    public Collection<Value> provideAll() {
        return this.provideAll(this.configuration);
    }

    /**
     * Provide all objects.
     *
     * @return the provided objects from this instance and its parents (unless those are masked).
     */
    protected abstract Collection<Value> provideAll(Configuration configuration);

    @Override
    public Iterator<Value> iterator() {
        return this.provideAll().iterator();
    }
}
